@@ -41,7 +41,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * msnd_pinnacle.c,v 1.7 2003/04/23 10:01:18 perex Exp
+ * msnd_pinnacle.c,v 1.9 2004/03/06 16:55:13 tiwai Exp
  *
  * 12-3-2000  Modified IO port validation  Steve Sycamore
  *
@@ -59,6 +59,7 @@
 #include <linux/config.h>
 #include <linux/version.h>
 #include <linux/module.h>
+#include <linux/interrupt.h>
 #include <linux/slab.h>
 #include <linux/types.h>
 #include <linux/delay.h>
@@ -463,7 +464,7 @@ static __inline__ void snd_msnd_eval_dsp_msg(register WORD wMessage)
 		case HIDSP_PLAY_UNDER:
 #endif
 		case HIDSP_INT_PLAY_UNDER:
-			printk(KERN_DEBUG LOGNAME ": Play underflow %i %lX\n", banksPlayed, xtime.tv_usec);
+			printk(KERN_DEBUG LOGNAME ": Play underflow %i\n", banksPlayed);
 			if( banksPlayed > 2)
 				clear_bit(F_WRITING, &dev.flags);
 			break;
