@@ -16,6 +16,8 @@
  ***************************************************************************/
 #define __NO_VERSION__
 #include <sound/driver.h>
+#include <linux/init.h>
+#include <asm/io.h>
 #include <sound/core.h>
 #include <sound/control.h>
 #include "msnd.h"
