@@ -1,8 +1,11 @@
 #include <linux/config.h>
 #include <linux/version.h>
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,5)
+#define SND_NEED_USB_SET_INTERFACE
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 #define SND_NEED_USB_WRAPPER
+#endif
 #define __NO_VERSION__
 #include <sound/driver.h>
 #include <linux/usb.h>
